% This code is designed to generate a training dataset and is already set up 
% to produce objects of different shapes. You can select the annotated content as needed.
% Before running the code, support files from EIDORS 3.11 with Netgen 5.3 are required. 
% Please refer to https://eidors3d.sourceforge.net/


%% one ball
mkdir absolute_oneball_1W_95295_v-max4.5 BV;
mkdir absolute_oneball_1W_95295_v-max4.5 BV_reshape;
mkdir absolute_oneball_1W_95295_v-max4.5 DDL;
% mkdir absolute_oneball_1W_95295_v-max4.5 fig;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_20dB;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_20dB_reshape;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_30dB;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_30dB_reshape;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_40dB;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_40dB_reshape;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_50dB;
mkdir absolute_oneball_1W_95295_v-max4.5\BV_50dB_reshape;

BV_dir = './absolute_oneball_1W_95295_v-max4.5/BV/';
BV_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_reshape/';
DDL_dir = './absolute_oneball_1W_95295_v-max4.5/DDL/';
% fig_dir = './absolute_oneball_1W_95295_v-max4.5/fig/';
BV_20dB_dir = './absolute_oneball_1W_95295_v-max4.5/BV_20dB/';
BV_20dB_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_20dB_reshape/';
BV_30dB_dir = './absolute_oneball_1W_95295_v-max4.5/BV_30dB/';
BV_30dB_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_30dB_reshape/';
BV_40dB_dir = './absolute_oneball_1W_95295_v-max4.5/BV_40dB/';
BV_40dB_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_40dB_reshape/';
BV_50dB_dir = './absolute_oneball_1W_95295_v-max4.5/BV_50dB/';
BV_50dB_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_50dB_reshape/';
% BV_80dB_dir = './absolute_oneball_1W_95295_v-max4.5/BV_80dB/';
% BV_80dB_reshape_dir = './absolute_oneball_1W_95295_v-max4.5/BV_80dB_reshape/';

bg = 0.035; % Background conductivity (water)
cond = 0.00000001; % Conductivity range of the substance to be measured (acrylic column)
num = 10000; % sample size

for k = 1:num
    cyl_shape = [0.08,0.05,0.004]; % % Dimensions of the domain to be measured [height, radius] 0.002 490354 nodes
    nelec= 16; % Number of electrodes per layer
    ring_vert_pos = [0.0196, 0.0392, 0.0588];  % Electrode ring position
    nrings= 3; % Number of electrode rings
    elec_shape = [0.001,0,0.02];  % Electrode shape [radius, 0, 0.001]
    fmdl= ng_mk_cyl_models(cyl_shape,[nelec,ring_vert_pos],elec_shape); 
    % show_fem(fmdl);
    
    % Establishing a generic EIT model
    imdl = mk_common_model('b3cr',[16,3]); % Will replace most fields
    imdl.fwd_model = fmdl;
    imdl.fwd_model.stimulation = mk_stim_patterns(nelec,nrings,[0,1],[0,1],{'no_meas_current','rotate_meas'},0.03); % 10mA
    img1 = mk_image(imdl, bg);  
    % show_fem(img1);
    vh = fwd_solve(img1);
    
    % Calculate element membership in object  
    %Sphere expression calculation
    x_d = unifrnd(-0.02,0.02);
    y_d = unifrnd(-0.02,0.02);
    z_d = unifrnd(0.02,0.05);
    r = unifrnd(0.01,0.02);

    expression = strcat('(x-',string(x_d),').^2+(y-',string(y_d),').^2+(z-',string(z_d),').^2<',string(r),'^2');
    select_fcn = inline(expression,'x','y','z');

    memb_frac = elem_select(img1.fwd_model, select_fcn);  
    img2 = mk_image(img1, memb_frac*cond );
    for i=1:95295
        if img2.elem_data(i)==0
            img2.elem_data(i)=bg;
        end
    end
    
    writematrix(vi.meas,strcat(BV_dir,string(k),'.csv'));
    vi_reshape = reshape(vi.meas,[45,48]);
    writematrix(vi_reshape,strcat(BV_reshape_dir,string(k),'.csv'));
    
   
    writematrix(img2.elem_data,strcat(DDL_dir,string(k),'_ddl.csv')); 

%     show_fem(img2,[0,0,0]); 
%     filename = strcat(fig_dir,string(k),'_',string(x_d),'_',string(y_d),'_',string(z_d),'_',string(r),'.fig');
%     saveas(gcf,filename);

%     SNRdB = 20; 
    vi_20dB = add_noise(10^(20/20) ,vi);
    writematrix(vi_20dB.meas,strcat(BV_20dB_dir,string(k),'.csv'));
    vi_20dB_reshape = reshape(vi_20dB.meas,[45,48]);
    writematrix(vi_20dB_reshape,strcat(BV_20dB_reshape_dir,string(k),'.csv'));

%     SNRdB = 30; 
    vi_30dB = add_noise(10^(30/20) ,vi);
    writematrix(vi_30dB.meas,strcat(BV_30dB_dir,string(k),'.csv'));
    vi_30dB_reshape = reshape(vi_30dB.meas,[45,48]);
    writematrix(vi_30dB_reshape,strcat(BV_30dB_reshape_dir,string(k),'.csv'));

%     SNRdB = 40; 
    vi_40dB = add_noise(10^(40/20) ,vi);
    writematrix(vi_40dB.meas,strcat(BV_40dB_dir,string(k),'.csv'));
    vi_40dB_reshape = reshape(vi_40dB.meas,[45,48]);
    writematrix(vi_40dB_reshape,strcat(BV_40dB_reshape_dir,string(k),'.csv'));

    %     SNRdB = 50; 
    vi_50dB = add_noise(10^(50/20) ,vi);
    writematrix(vi_50dB.meas,strcat(BV_50dB_dir,string(k),'.csv'));
    vi_50dB_reshape = reshape(vi_50dB.meas,[45,48]);
    writematrix(vi_50dB_reshape,strcat(BV_50dB_reshape_dir,string(k),'.csv'));

% %     SNRdB = 80; 
%     vi_80dB = add_noise(10^( 80/20) ,vi);
%     writematrix(vi_80dB.meas,strcat(BV_80dB_dir,string(k),'.csv'));
%     vi_80dB_reshape = reshape(vi_80dB.meas,[45,48]);
%     writematrix(vi_80dB_reshape,strcat(BV_80dB_reshape_dir,string(k),'.csv'));

end



% %% one prism
% 
% mkdir absolute_oneprism_1W_95295_v-max4.5 BV;
% mkdir absolute_oneprism_1W_95295_v-max4.5 BV_reshape;
% mkdir absolute_oneprism_1W_95295_v-max4.5 DDL;
% % mkdir absolute_oneprism_1W_95295_v-max4.5 fig;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_20dB;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_20dB_reshape;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_30dB;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_30dB_reshape;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_40dB;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_40dB_reshape;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_50dB;
% mkdir absolute_oneprism_1W_95295_v-max4.5\BV_50dB_reshape;
% 
% BV_dir = './absolute_oneprism_1W_95295_v-max4.5/BV/';
% BV_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_reshape/';
% DDL_dir = './absolute_oneprism_1W_95295_v-max4.5/DDL/';
% % fig_dir = './absolute_oneprism_1W_95295_v-max4.5/fig/';
% BV_20dB_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_20dB/';
% BV_20dB_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_20dB_reshape/';
% BV_30dB_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_30dB/';
% BV_30dB_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_30dB_reshape/';
% BV_40dB_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_40dB/';
% BV_40dB_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_40dB_reshape/';
% BV_50dB_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_50dB/';
% BV_50dB_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_50dB_reshape/';
% % BV_80dB_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_80dB/';
% % BV_80dB_reshape_dir = './absolute_oneprism_1W_95295_v-max4.5/BV_80dB_reshape/';
% 
% bbg = 0.035; 
% cond = 0.00000001; 
% num = 10000; 
% 
% for k = 1:num
%     cyl_shape = [0.08,0.05,0.004]; 
%     nelec= 16;
%     ring_vert_pos = [0.0196, 0.0392, 0.0588];
%     nrings= 3; 
%     elec_shape = [0.001,0,0.02]; 

%     fmdl= ng_mk_cyl_models(cyl_shape,[nelec,ring_vert_pos],elec_shape); 
%     % show_fem(fmdl);
%     
%     imdl = mk_common_model('b3cr',[16,3]); % Will replace most fields
%     imdl.fwd_model = fmdl;

%     imdl.fwd_model.stimulation = mk_stim_patterns(nelec,nrings,[0,1],[0,1],{'no_meas_current','rotate_meas'},0.03); % 10mA
%     img1 = mk_image(imdl, bg);  
%     % show_fem(img1);
%     vh = fwd_solve(img1);
%     
%     % Calculate element membership in object  
%     % % Cylindrical expression calculation
%     x_d = unifrnd(-0.018,0.018);
%     y_d = unifrnd(-0.018,0.018);
%     r = unifrnd(0.01,0.02);
% 
%     expression = strcat('abs(x-',string(x_d),')+abs(y-',string(y_d),')<',string(r));
%     select_fcn = inline(expression,'x','y','z');
%     
%     memb_frac = elem_select(img1.fwd_model, select_fcn);  
%     img2 = mk_image(img1, memb_frac*cond );
%     for i=1:95295
%         if img2.elem_data(i)==0
%             img2.elem_data(i)=bg;
%         end
%     end
% 
%     vi=fwd_solve(img2);
% 

%     writematrix(vi.meas,strcat(BV_dir,string(k),'.csv'));
%     vi_reshape = reshape(vi.meas,[45,48]);
%     writematrix(vi_reshape,strcat(BV_reshape_dir,string(k),'.csv'));
%     
%    
%     writematrix(img2.elem_data,strcat(DDL_dir,string(k),'_ddl.csv')); 
% 
% %     SNRdB = 20;
%     vi_20dB = add_noise(10^(20/20) ,vi);
%     writematrix(vi_20dB.meas,strcat(BV_20dB_dir,string(k),'.csv'));
%     vi_20dB_reshape = reshape(vi_20dB.meas,[45,48]);
%     writematrix(vi_20dB_reshape,strcat(BV_20dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 30;
%     vi_30dB = add_noise(10^(30/20) ,vi);
%     writematrix(vi_30dB.meas,strcat(BV_30dB_dir,string(k),'.csv'));
%     vi_30dB_reshape = reshape(vi_30dB.meas,[45,48]);
%     writematrix(vi_30dB_reshape,strcat(BV_30dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 40;
%     vi_40dB = add_noise(10^(40/20) ,vi);
%     writematrix(vi_40dB.meas,strcat(BV_40dB_dir,string(k),'.csv'));
%     vi_40dB_reshape = reshape(vi_40dB.meas,[45,48]);
%     writematrix(vi_40dB_reshape,strcat(BV_40dB_reshape_dir,string(k),'.csv'));
% 
%     %     SNRdB = 50;
%     vi_50dB = add_noise(10^(50/20) ,vi);
%     writematrix(vi_50dB.meas,strcat(BV_50dB_dir,string(k),'.csv'));
%     vi_50dB_reshape = reshape(vi_50dB.meas,[45,48]);
%     writematrix(vi_50dB_reshape,strcat(BV_50dB_reshape_dir,string(k),'.csv'));
% 
% end


% %% cylinder
% 
% mkdir absolute_onecylindrical_1W_95295_v-max4.5 BV;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5 BV_reshape;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5 DDL;
% % mkdir absolute_onecylindrical_1W_95295_v-max4.5 fig;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_20dB;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_20dB_reshape;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_30dB;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_30dB_reshape;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_40dB;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_40dB_reshape;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_50dB;
% mkdir absolute_onecylindrical_1W_95295_v-max4.5\BV_50dB_reshape;
% 
% BV_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV/';
% BV_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_reshape/';
% DDL_dir = './absolute_onecylindrical_1W_95295_v-max4.5/DDL/';
% % fig_dir = './absolute_onecylindrical_1W_95295_v-max4.5/fig/';
% BV_20dB_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_20dB/';
% BV_20dB_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_20dB_reshape/';
% BV_30dB_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_30dB/';
% BV_30dB_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_30dB_reshape/';
% BV_40dB_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_40dB/';
% BV_40dB_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_40dB_reshape/';
% BV_50dB_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_50dB/';
% BV_50dB_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_50dB_reshape/';
% % BV_80dB_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_80dB/';
% % BV_80dB_reshape_dir = './absolute_onecylindrical_1W_95295_v-max4.5/BV_80dB_reshape/';
% 
% bg = 0.035; 
% cond = 0.00000001; 
% num = 10000;
% 
% for k = 1:num
%     cyl_shape = [0.08,0.05,0.004];
%     nelec= 16; 
%     ring_vert_pos = [0.0196, 0.0392, 0.0588];  
%     nrings= 3; 
%     elec_shape = [0.001,0,0.02]; 

%     fmdl= ng_mk_cyl_models(cyl_shape,[nelec,ring_vert_pos],elec_shape); 
%     % show_fem(fmdl);
%     

%     imdl = mk_common_model('b3cr',[16,3]); % Will replace most fields
%     imdl.fwd_model = fmdl;

%     imdl.fwd_model.stimulation = mk_stim_patterns(nelec,nrings,[0,1],[0,1],{'no_meas_current','rotate_meas'},0.03); % 10mA
%     img1 = mk_image(imdl, bg); 
%     % show_fem(img1);
%     vh = fwd_solve(img1);
%     
%     % Calculate element membership in object 

%     x_d = unifrnd(-0.018,0.018);
%     y_d = unifrnd(-0.018,0.018);
%     z_d = unifrnd(0.02,0.05);
%     r = unifrnd(0.01,0.02);
% 
% %     expression = strcat('(x-',string(x_d),').^2+(y-',string(y_d),').^2+(z-',string(z_d),').^2<',string(r),'^2');
%     expression = strcat('(x-',string(x_d),').^2+(y-',string(y_d),').^2<',string(r),'^2');
% %     select_fcn = inline(expression,'x','y','z');
%     select_fcn = inline(expression,'x','y','z');
%     
%     memb_frac = elem_select(img1.fwd_model, select_fcn); 
%     img2 = mk_image(img1, memb_frac*cond );
%     for i=1:95295
%         if img2.elem_data(i)==0
%             img2.elem_data(i)=bg;
%         end
%     end  
% 
%     vi=fwd_solve(img2);
% 

%     writematrix(vi.meas,strcat(BV_dir,string(k),'.csv'));
%     vi_reshape = reshape(vi.meas,[45,48]);
%     writematrix(vi_reshape,strcat(BV_reshape_dir,string(k),'.csv'));
%     
%    
%     writematrix(img2.elem_data,strcat(DDL_dir,string(k),'_ddl.csv')); 
% 
% %     SNRdB = 20; 
%     vi_20dB = add_noise(10^(20/20) ,vi);
%     writematrix(vi_20dB.meas,strcat(BV_20dB_dir,string(k),'.csv'));
%     vi_20dB_reshape = reshape(vi_20dB.meas,[45,48]);
%     writematrix(vi_20dB_reshape,strcat(BV_20dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 30; 
%     vi_30dB = add_noise(10^(30/20) ,vi);
%     writematrix(vi_30dB.meas,strcat(BV_30dB_dir,string(k),'.csv'));
%     vi_30dB_reshape = reshape(vi_30dB.meas,[45,48]);
%     writematrix(vi_30dB_reshape,strcat(BV_30dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 40; 
%     vi_40dB = add_noise(10^(40/20) ,vi);
%     writematrix(vi_40dB.meas,strcat(BV_40dB_dir,string(k),'.csv'));
%     vi_40dB_reshape = reshape(vi_40dB.meas,[45,48]);
%     writematrix(vi_40dB_reshape,strcat(BV_40dB_reshape_dir,string(k),'.csv'));
% 
%     %     SNRdB = 50; 
%     vi_50dB = add_noise(10^(50/20) ,vi);
%     writematrix(vi_50dB.meas,strcat(BV_50dB_dir,string(k),'.csv'));
%     vi_50dB_reshape = reshape(vi_50dB.meas,[45,48]);
%     writematrix(vi_50dB_reshape,strcat(BV_50dB_reshape_dir,string(k),'.csv'));
% 
% end



% %% ball,prism,and cylinder
% 
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test BV;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test BV_reshape;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test DDL;
% % mkdir absolute_threeobj_1W_95295_v-max_4.5_test fig;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_20dB;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_20dB_reshape;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_30dB;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_30dB_reshape;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_40dB;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_40dB_reshape;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_50dB;
% mkdir absolute_threeobj_1W_95295_v-max_4.5_test\BV_50dB_reshape;
% 
% BV_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV/';
% BV_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_reshape/';
% DDL_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/DDL/';
% % fig_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/fig/';
% BV_20dB_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_20dB/';
% BV_20dB_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_20dB_reshape/';
% BV_30dB_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_30dB/';
% BV_30dB_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_30dB_reshape/';
% BV_40dB_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_40dB/';
% BV_40dB_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_40dB_reshape/';
% BV_50dB_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_50dB/';
% BV_50dB_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_50dB_reshape/';
% % BV_80dB_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_80dB/';
% % BV_80dB_reshape_dir = './absolute_threeobj_1W_95295_v-max_4.5_test/BV_80dB_reshape/';
% 
% bg = 0.035; 
% cond = 0.00000001; 
% num = 100; 
% 
% for k = 1:num
%     cyl_shape = [0.08,0.05,0.004]; 
%     nelec= 16; 
%     ring_vert_pos = [0.0196, 0.0392, 0.0588];
%     nrings= 3; 
%     elec_shape = [0.001,0,0.02];  

%     fmdl= ng_mk_cyl_models(cyl_shape,[nelec,ring_vert_pos],elec_shape); 
%     % show_fem(fmdl);
%     

%     imdl = mk_common_model('b3cr',[16,3]); % Will replace most fields
%     imdl.fwd_model = fmdl;

%     imdl.fwd_model.stimulation = mk_stim_patterns(nelec,nrings,[0,1],[0,1],{'no_meas_current','rotate_meas'},0.03); % 10mA
%     img1 = mk_image(imdl, bg); 
%     % show_fem(img1);
%     vh = fwd_solve(img1);
% 
%  
%     t = randi([0,23]);
%     disp(k)
%     switch t
%         case 0
%             %Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
% 
%             %Q3
%             x_d_r = unifrnd(-0.02,-0.015);
%             y_d_r = unifrnd(-0.02,-0.015);
%             r_r = unifrnd(0.01,0.015);
% 
%         case 1
%             %Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
% 
%             %Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 2
% %             %    Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 3
%             %    Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
% 
%         case 4
%             %    Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%        case 5
%             %    Q1
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 6
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 7
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 8
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 9
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 10
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
% 
%             %    Q3
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 11
%             %    Q2
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(0.015,0.02);
%             z_d_b = unifrnd(0.025,0.055);
%             r_b = unifrnd(0.01,0.015);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 12
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 13
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 14
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 15
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q4
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 16
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 17
%             %    Q3
%             x_d_b = unifrnd(-0.02,-0.015);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q4
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 18
%             %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q3
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 19
%             %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q1
%             x_d_p = unifrnd(0.015,0.02);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 20
%             %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 21
%              %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q2
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(0.015,0.02);
%             r_p = unifrnd(0.01,0.015);
%             %    Q3
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(-0.02,-0.015);
%             r_c = unifrnd(0.01,0.015);
%         case 22
%             %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
%             %    Q2
%             x_d_c = unifrnd(-0.02,-0.015);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%         case 23
%             %    Q4
%             x_d_b = unifrnd(0.015,0.02);
%             y_d_b = unifrnd(-0.02,-0.015);
%             r_b = unifrnd(0.01,0.015);
%             z_d_b = unifrnd(0.025,0.055);
% 
%             %    Q3
%             x_d_p = unifrnd(-0.02,-0.015);
%             y_d_p = unifrnd(-0.02,-0.015);
%             r_p = unifrnd(0.01,0.015);
%             %    Q1
%             x_d_c = unifrnd(0.015,0.02);
%             y_d_c = unifrnd(0.015,0.02);
%             r_c = unifrnd(0.01,0.015);
%     end
% 
% 
% 
%     expression_1 = strcat('(x-',string(x_d_b),').^2+(y-',string(y_d_b),').^2+(z-',string(z_d_b),').^2<',string(r_b),'^2');
%     select_fcn_1 = inline(expression_1,'x','y','z');
% 
%     memb_frac_1 = elem_select(img1.fwd_model, select_fcn_1); 
% 
% 
% 
%     expression_2 = strcat('abs(x-',string(x_d_p),')+abs(y-',string(y_d_p),')<',string(r_p));
%     select_fcn_2 = inline(expression_2,'x','y','z');
%     
%     memb_frac_2 = elem_select(img1.fwd_model, select_fcn_2); 
% 
% 
%     expression_3 = strcat('(x-',string(x_d_c),').^2+(y-',string(y_d_c),').^2<',string(r_c),'^2');
%     select_fcn_3 = inline(expression_3,'x','y','z');
%     
%     memb_frac_3 = elem_select(img1.fwd_model, select_fcn_3);  
% 
% 
%     memb_frac = memb_frac_1 + memb_frac_2 + memb_frac_3;
% 
%     img2 = mk_image(img1, memb_frac*cond );
%     for i=1:95295
%         if img2.elem_data(i)==0
%             img2.elem_data(i)=bg;
%         end
%     end
% 
%     vi=fwd_solve(img2);
%     show_fem(img2,[1,1])
% 

%     writematrix(vi.meas,strcat(BV_dir,string(k),'.csv'));
%     vi_reshape = reshape(vi.meas,[45,48]);
%     writematrix(vi_reshape,strcat(BV_reshape_dir,string(k),'.csv'));
%     writematrix(img2.elem_data,strcat(DDL_dir,string(k),'_ddl.csv'));
% 
% %     SNRdB = 20; 
%     vi_20dB = add_noise(10^(20/20) ,vi);
%     writematrix(vi_20dB.meas,strcat(BV_20dB_dir,string(k),'.csv'));
%     vi_20dB_reshape = reshape(vi_20dB.meas,[45,48]);
%     writematrix(vi_20dB_reshape,strcat(BV_20dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 30; 
%     vi_30dB = add_noise(10^(30/20) ,vi);
%     writematrix(vi_30dB.meas,strcat(BV_30dB_dir,string(k),'.csv'));
%     vi_30dB_reshape = reshape(vi_30dB.meas,[45,48]);
%     writematrix(vi_30dB_reshape,strcat(BV_30dB_reshape_dir,string(k),'.csv'));
% 
% %     SNRdB = 40;
%     vi_40dB = add_noise(10^(40/20) ,vi);
%     writematrix(vi_40dB.meas,strcat(BV_40dB_dir,string(k),'.csv'));
%     vi_40dB_reshape = reshape(vi_40dB.meas,[45,48]);
%     writematrix(vi_40dB_reshape,strcat(BV_40dB_reshape_dir,string(k),'.csv'));
% 
%     %     SNRdB = 50; 
%     vi_50dB = add_noise(10^(50/20) ,vi);
%     writematrix(vi_50dB.meas,strcat(BV_50dB_dir,string(k),'.csv'));
%     vi_50dB_reshape = reshape(vi_50dB.meas,[45,48]);
%     writematrix(vi_50dB_reshape,strcat(BV_50dB_reshape_dir,string(k),'.csv'));
% 
% 
% end
% 
















