% The code provided here offers a method for reconstructing conductivity into images, 
% primarily focusing on imaging of simulated data and acrylic data.
% "Before running the code, support files from EIDORS 3.11 with Netgen 5.3 are required. Please refer to https://eidors3d.sourceforge.net/"

clc;
clear;

DDL_name='DDL_threeobj_None_ResNet';  
Output_name='Output_threeobj_None_ResNet';  
score = 'Score_None_ResNet'; 

dir = 'D:\Projects\3DEIT\output\predict_threeobj_ResNet';  
mkdir D:\Projects\3DEIT\output\predict_threeobj_ResNet\ Figure_threeobj_None_ResNet          
save_fig = 'D:\Projects\3DEIT\output\predict_threeobj_ResNet\Figure_threeobj_None_ResNet';    



rmses = [];
corrs = [];
ssims = [];
                                                       
num = 100;  
bg = 0.035; 

cyl_shape = [0.08,0.05,0.004]; 
nelec= 16; 
ring_vert_pos = [0.0196, 0.0392, 0.0588];
nrings= 3; 
elec_shape = [0.001,0,0.02];  

fmdl= ng_mk_cyl_models(cyl_shape,[nelec,ring_vert_pos],elec_shape); 

imdl = mk_common_model('b3cr',[16,3]); 
imdl.fwd_model = fmdl;

imdl.fwd_model.stimulation = mk_stim_patterns(nelec,nrings,[0,1],[0,1],{'no_meas_current','rotate_meas'},0.03);
img1 = mk_image(imdl, bg);  
show_fem(img1,3);

for i=1:num

    i
    img2 = img1; 
    sig2 = readmatrix(strcat(dir,'\',DDL_name,'\',num2str(i),'_ddl.csv'));
    img2.elem_data = sig2 ;

    subplot(2,2,1);
    show_fem(img2);
    eidors_colourbar(img2)  %colourbar
    title('Target')
    
    subplot(2,2,2);
    show_3d_slices(img2,[0.02,0.04,0.06]);
    title('Target slices')
    img3 = img1;
    sig3 = readmatrix(strcat(dir,'\',Output_name,'\',num2str(i),'_ddl.csv'));
    img3.elem_data = sig3;
    subplot(2,2,3);
    show_fem(img3);
    title('Reconstruction')
    
    subplot(2,2,4);
    show_3d_slices(img3,[0.02,0.04,0.06])
    title('Reconstruction slices')
    saveas(gcf,strcat(save_fig,'\',num2str(i),'.fig'))
    saveas(gcf,strcat(save_fig,'\',num2str(i),'.png'))

    % rmse = sqrt(mean((YPred-YReal).^2));
    rmse1 = rmse(img3.elem_data,img2.elem_data);
    rmses = [rmses,rmse1];  % RMSE

    corr1 = corrcoef(img3.elem_data, img2.elem_data);

    corrs =[corrs,corr1(1,2)];  % CC

    ssim1 = ssim(img3.elem_data,img2.elem_data);
    ssims = [ssims,ssim1];  % SSIM

end


rmses = rmses';
corrs = corrs';
ssims = ssims';
value = [rmses,corrs,ssims];
score_name = {'rmses','corrs','ssims'};
writecell(score_name,strcat(dir,'\',score,'.xlsx'));
writematrix(value,strcat(dir,'\',score,'.xlsx'),'WriteMode','append');


rmses_mean = mean(rmses);
rmses_std = std(rmses);
corrs_mean = mean(corrs);
corrs_std = std(corrs);
ssims_mean = mean(ssims);
ssims_std = std(ssims);

writeT = {'rmses_mean:',rmses_mean;
    'rmses_std:',rmses_std;
    'corrs_mean:',corrs_mean;
    'corrs_std:',corrs_std;
    'ssims_mean:',ssims_mean; 
    'ssims_std:',ssims_std};

writecell(writeT,strcat(dir,'\',score,'.txt')) 



